/**
 Copyright (c) 2015-present, Facebook, Inc.
 All rights reserved.

 This source code is licensed under the BSD-style license found in the
 LICENSE file in the root directory of this source tree. An additional grant
 of patent rights can be found in the PATENTS file in the same directory.
 */

#ifndef __pbxbuild_Build_Environment_h
#define __pbxbuild_Build_Environment_h

#include <pbxbuild/Base.h>

#include <ext/optional>

namespace pbxbuild {
namespace Build {

/*
 * Represents the collection of utilities needed for a build. These utilties
 * are not tied to the build but are used across the build.
 */
class Environment {
private:
    pbxspec::Manager::shared_ptr         _specManager;
    std::shared_ptr<xcsdk::SDK::Manager> _sdkManager;
    pbxsetting::Environment              _baseEnvironment;

public:
    Environment(
        pbxspec::Manager::shared_ptr const &specManager,
        std::shared_ptr<xcsdk::SDK::Manager> const &sdkManager,
        pbxsetting::Environment const &baseEnvironment);

public:
    /*
     * The specification manager and specifications for the build.
     */
    pbxspec::Manager::shared_ptr const &specManager() const
    { return _specManager; }

    /*
     * The SDK manager for the build.
     */
    std::shared_ptr<xcsdk::SDK::Manager> const &sdkManager() const
    { return _sdkManager; }

public:
    /*
     * The base environment from the system.
     */
    pbxsetting::Environment const &baseEnvironment() const
    { return _baseEnvironment; }

public:
    /*
     * Creates a build environment from the default configuration
     * of each of the build environment's subcomponents.
     */
    static ext::optional<Environment>
    Default(void);
};

}
}

#endif // !__pbxbuild_Build_Environment_h
